import {interfaces} from "inversify";
import {CommandBase} from "./commands/core/CommandBase";
import {CommandArguments} from "./commands/core/CommandArguments";
import {FrameworkBootstrapper, LogLevel, LoggingService, LoggingServiceFactory} from "nodevolution-framework";
import {CLIModule} from "./CLIModule";
import {CommandFactory} from "./commands/CommandFactory";
import {ConsoleLoggingPipeline} from "./logging/ConsoleLoggingPipeline";


export class CLI {
    private _container: interfaces.Container;
    private _commandFactory: CommandFactory;
    private _logger: LoggingService;

    public initialize(logLevel: LogLevel): Promise<void> {
        let bootstrapper = new FrameworkBootstrapper();
        bootstrapper
            .withIocContainer()
            .withLogging(logLevel, [new ConsoleLoggingPipeline(logLevel)])
            .bootstrap();

        this._container = bootstrapper.getContainer();
        this._container.load(new CLIModule());

        this._commandFactory = this._container.get<CommandFactory>("CommandFactory");
        this._logger = this.createLogger();
        return Promise.resolve();
    }

    public run(commandName: string, args: Array<string>): Promise<any> {
        this._logger.debug(`try to instanciate command "${commandName}"`, {command: commandName, args: args});
        let command = this.createCommand(commandName);
        this._logger.debug(`found command, will now run it...`);
        return command.execute(new CommandArguments(args));
    }

    private createCommand(commandName: string): CommandBase {
        return this._commandFactory.create(this._container, commandName);
    }

    private createLogger(): LoggingService {
        let factory = this._container.get<LoggingServiceFactory>("LoggingServiceFactory");
        return factory.create("NXCLI");
    }
}