import * as inquirer from "inquirer";
import Answers = inquirer.Answers;
import Question = inquirer.Question;
import * as Process from "process";
import * as Path from "path";
import {
    LoggingServiceFactory,
    LoggingService,
    ConfigurationService,
    VersionService,
    PackageService,
    BumpPackageCommand,
    BumpDependencyCommand,
    BumpDevDependencyCommand, NxConfiguration, PackageInfo, VersionLevel, BumpInfo
} from "nodevolution-framework";
import {inject, injectable} from "inversify";
import {ArgumentHandlerBase} from "../../core/ArgumentHandlerBase";
import {CommandArguments} from "../../core/CommandArguments";

@injectable()
export class BumpArgumentHandler extends ArgumentHandlerBase {

    private _configurationService: ConfigurationService;
    private _versionService: VersionService;
    private _loggingService: LoggingService;
    private _packageService: PackageService;

    private _configuration: NxConfiguration;
    private _packages: Array<PackageInfo>;

    private _packageName: string;
    private _versionLevelChoice: string;

    constructor(@inject("LoggingServiceFactory") loggingServiceFactory: LoggingServiceFactory,
                @inject("PackageService") packageService: PackageService,
                @inject("ConfigurationService") configurationService: ConfigurationService,
                @inject("VersionService") versionService: VersionService) {
        super();
        this._packageService = packageService;
        this._configurationService = configurationService;
        this._versionService = versionService;
        this._loggingService = loggingServiceFactory.create("LinkArgumentHandler");
    }

    public initialize(): Promise<any> {
        return new Promise<any>((resolve) => {
            resolve();
        });
    }

    public async prepare(args: CommandArguments): Promise<any> {
        let workingDirectory = Process.cwd();
        this._loggingService.debug(`detected working directory ${workingDirectory}`);
        this._configuration = await this.loadConfiguration(workingDirectory);
        this._packages = this._packageService.locatePackages(this._configuration.bump.files);

        let askForPackage = true;
        let askForVersionLevel = true;

        let packageName = args.next(args.get("bump"));
        if (packageName != undefined) {
            this._packageName = packageName.key;
            askForPackage = false;
        }

        let versionLevel = args.next(packageName);
        if (versionLevel != undefined) {
            this._versionLevelChoice = versionLevel.key;
            askForVersionLevel = false;
        }
        let questions: Array<Question> = [
            {
                name: "packageName",
                message: "choose the package to bump",
                type: "list",
                choices: this._packages.map((x) => x.name),
                when: askForPackage
            },
            {
                name: "versionLevel",
                message: "choose the version level",
                type: "list",
                choices: this.getVersionLevelChoices(),
                when: askForVersionLevel
            }
        ];
        return await inquirer.prompt(questions);
    }

    public async handle(args: CommandArguments, answers: Answers): Promise<any> {
        this._loggingService.debug("start bump command");

        if (this._packageName == undefined) {
            this._packageName = answers["packageName"];
            if (this._packageName == undefined) {
                this._loggingService.error("package name was not specified");
                throw new Error("package name was not specified");
            }
        }

        if (this._versionLevelChoice == undefined) {
            this._versionLevelChoice = answers["versionLevel"];
            if (this._versionLevelChoice == undefined) {
                this._loggingService.error("version level was not specified");
                throw new Error("version level was not specified");
            }
        }

        let versionLevel = this._versionService.getVersionLevel(this._versionLevelChoice);
        if (versionLevel == undefined) {
            this._loggingService.error(`version level could not be mapped (${this._versionLevelChoice})`, this._versionLevelChoice);
            throw new Error(`version level could not be mapped (${this._versionLevelChoice})`);
        }
        this._loggingService.debug(`mapped version level to ${versionLevel}`, versionLevel);


        let initialPackage = this._packages.find((x) => {
            return x.name === this._packageName;
        });

        if (initialPackage == undefined) {
            this._loggingService.error(`specified package (${this._packageName}) to bump was not found.`, this._packageName);
            throw new Error(`specified package (${this._packageName}) to bump was not found.`);
        }

        let bumpInfos = await this._versionService.createVersinBumpInfos(initialPackage, versionLevel, this._packages);
        bumpInfos.forEach((info) => {
            info.commands.forEach((cmd) => {
                if (cmd instanceof BumpPackageCommand) {
                    this._loggingService.info(`${cmd.packageName}: ${cmd.version} => ${cmd.newVersion}`);
                }
                else if (cmd instanceof BumpDependencyCommand) {
                    this._loggingService.info(`\t${cmd.dependencyPackageName} ${cmd.version} => ${cmd.newVersion}`);
                }
                else if (cmd instanceof BumpDevDependencyCommand) {
                    this._loggingService.info(`\t${cmd.dependencyPackageName} ${cmd.version} => ${cmd.newVersion}`);
                }
            });
        });

        if (args.hasOption("f")) {
            await this.persistChanges(bumpInfos);
        }
        else {
            let questions: Array<Question> = [
                {name: "persist", type: "confirm", message: "persist changes?"}
            ];
            let answers = await inquirer.prompt(questions);
            if (answers["persist"] === true) {
                await this.persistChanges(bumpInfos);
            }
            else {
                this._loggingService.warn(`aborted. changes not persited`);
            }
        }


    }

    private async persistChanges(bumpInfos: Array<BumpInfo>): Promise<void> {
        await this._versionService.persistChanges(bumpInfos);
        this._loggingService.info(`persisted changes`);
    }

    private async loadConfiguration(workingDirectory: string): Promise<NxConfiguration> {
        let configurationFilePath = this._configurationService.search(workingDirectory);
        if (configurationFilePath === undefined) {
            this._loggingService.error(`no configuration file was detected. exiting`);
            return;
        }
        this._loggingService.debug(`loading configuration file from location ${configurationFilePath}`);
        let configuration = await this._configurationService.load(configurationFilePath);
        this._loggingService.debug(`loaded configuration file`, configuration);

        if (configuration.bump == undefined) {
            this._loggingService.error(`configuration file has no bump section. Please create bump section with files.`, configuration);
            return;
        }
        if (configuration.bump.files == undefined) {
            this._loggingService.error(`bump section has no files defined. Please specify search directories in files section.`, configuration);
            return;
        }

        return configuration;
    }

    private getVersionLevelChoices(): Array<string> {
        let versionLevelChoices: Array<string> = [];
        for (let value in VersionLevel) {
            if (typeof VersionLevel[value] === "number") {
                versionLevelChoices.push(value);
            }
        }
        return versionLevelChoices;
    }
}